# 将本模块加入到目标类里，可生成类方法和实例方法 logger，它返回绑定到目标类的日志。
require 'logger'

module SpiderFramework::EnhancedLogger

  @level = ::Logger::DEBUG

  class << self
    attr_accessor :level, :names_only

    def included(base)
      # 构造绑定类的 @logger 实例
      base.class_eval do
        name = base.name
        names_only = EnhancedLogger.names_only
        if !names_only || names_only.include?(name)
          @logger = Logger.new(STDOUT)
          @logger.level = EnhancedLogger.level
          original_formatter = Logger::Formatter.new
          @logger.formatter = proc { |severity, datetime, progname, msg|
            "[#{datetime.strftime('%F %T')}] #{severity} -- #{name}: #{msg}\n"
          }
        else
          @logger = Logger.new(IO::NULL)
        end
      end

      base.singleton_class.class_eval do
        attr_accessor :logger
      end
    end
  end

  def logger
    self.class.logger
  end
end
